/*      EVAR.H: Environment and user variable definitions
                for MicroEMACS

                written 1993 by Daniel Lawrence
*/

#define MAXVARS         512     /* Maximum number of global user variables */

/*      list of recognized environment variables        */

NOSHARE char *envars[] = {
        "abbell",               /* ring bell on abbreviation expansion? */
        "abcap",                /* match capitolization in expansions */
        "abquick",              /* quick, aggressive expansions enabled? */
        "acount",               /* # of chars until next auto-save */
        "asave",                /* # of chars between auto-saves */
        "bufhook",              /* enter buffer switch hook */
        "cbflags",              /* current buffer flags */
        "cbufname",             /* current buffer name */
        "cfname",               /* current file name */
        "cmdhook",              /* command loop hook */
        "cmode",                /* mode of current buffer */
        "curchar",              /* current character under the cursor */
        "curcol",               /* current column pos of cursor */
        "curline",              /* current line in file */
        "curwidth",             /* current screen width */
        "curwind",              /* current window ordinal on current screen */
        "cwline",               /* current screen line in window */
        "debug",                /* macro debugging */
        "deskcolor",            /* desktop color */
        "diagflag",             /* diagonal mouse movements enabled? */
        "discmd",               /* display commands on command line */
        "disinp",               /* display command line input characters */
        "disphigh",             /* display high bit characters escaped */
        "dispundo",             /* display undo depth on command line */
        "exbhook",              /* exit buffer switch hook */
        "exithook",             /* exiting emacs hook */
        "fcol",                 /* first displayed column in curent window */
        "fillcol",              /* current fill column */
        "flicker",              /* flicker supression */
        "fmtlead",              /* format command lead characters */
        "gflags",               /* global internal emacs flags */
        "gmode",                /* global modes */
        "hardtab",              /* current hard tab size */
        "hilight",              /* region # to hilight (255 to turn off) */
        "hilight1",             /* syntax highlight #1 attributes */
        "hilight2",             /* syntax highlight #2 attributes */
        "hilight3",             /* syntax highlight #3 attributes */
        "hilight4",             /* syntax highlight #4 attributes */
        "hjump",                /* horizontal screen jump size */
        "hscrlbar",             /* horizontal scroll bar flag */
        "hscroll",              /* horizontal scrolling flag */
        "isterm",               /* incremental-search terminator character */
        "kill",                 /* kill buffer (read only) */
        "language",             /* language of text messages */
        "lastkey",              /* last keyboard char struck */
        "lastmesg",             /* last string mlwrite()ed */
        "line",                 /* text of current line */
        "lterm",                /* current line terminator for writes */
        "lwidth",               /* width of current line */
        "match",                /* last matched magic pattern */
        "mmove",                /* mouse moves events style */
        "modeflag",             /* Modelines displayed flag */
        "msflag",               /* activate mouse? */
        "newscreen",            /* new screen with new buffer? */
        "numwind",              /* number of windows on current screen */
        "orgcol",               /* screen origin column */
        "orgrow",               /* screen origin row */
        "os",                   /* what Operating System? */
        "overlap",              /* line overlap when paging */
        "pagelen",              /* number of lines used by editor */
        "palette",              /* current palette string */
        "paralead",             /* paragraph leadin characters */
        "pending",              /* type ahead pending flag */
        "popflag",              /* pop-up windows active? */
        "popwait",              /* user wait on end of pop-up window? */
        "posflag",              /* display point position on modeline? */
        "progname",             /* returns current prog name - "MicroEMACS" */
        "ram",                  /* ram in use by malloc */
        "readhook",             /* read file execution hook */
        "region",               /* current region (read only) */
        "replace",              /* replacement pattern */
        "rval",                 /* child process return value */
        "scrname",              /* current screen name */
        "search",               /* search pattern */
        "searchpnt",            /* differing search styles (term point) */
        "seed",                 /* current random number seed */
        "shlhook",              /* syntax highlight hook */
        "softtab",              /* current soft tab size */
        "sres",                 /* current screen resolution */
        "ssave",                /* safe save flag */
        "sscroll",              /* smooth scrolling flag */
        "status",               /* returns the status of the last command */
        "sterm",                /* search terminator character */
        "syntax",               /* syntax type for highlighting */
        "target",               /* target for line moves */
        "time",                 /* date and time */
        "timeflag",             /* display time? */
        "tpause",               /* length to pause for paren matching */
        "undoflag",             /* currently processing undos */
        "version",              /* current version number */
        "vscrlbar",             /* vertical scroll bar flag */
        "wchars",               /* set of characters legal in words */
        "wline",                /* # of lines in current window */
        "wraphook",             /* wrap word execution hook */
        "writehook",            /* write file hook */
        "xpos",                 /* current mouse X position */
        "yankflag",             /* point placement at yanked/included text */
        "ypos"                  /* current mouse Y position */
#if QDOS
        ,
        "beep",                /* beep flag */
        "boldoff",             /* bold mode off printer codes */
        "boldon",              /* bold mode on printer codes */
        "cmdink",              /* command line ink */
        "cmdpaper",            /* command line paper */
        "crlf",                /* flag: CR+LF at end of line when printing */
        "dictfile",            /* default QTyp II dictionary file */
        "duration",            /* beep duration */
        "italoff",             /* italic off printer codes */
        "italon",              /* italic on printer codes */
        "mlink",               /* mode line ink */
        "mlpaper",             /* mode line paper */
        "pageflag",            /* flag: print page number */
        "pagesize",            /* page size in lines */
        "pitch",               /* beep pitch */
        "printer",             /* printer device name */
        "prtinit",             /* printer initialisation codes */
        "title",               /* title to be printed into the header */
        "translat",            /* translate chars to IBM PC-8 when printing */
        "underoff",            /* underlined off printer codes */
        "underon"              /* underlined on printer codes */
#endif
};

#define NEVARS  sizeof(envars) / sizeof(char *)

/*      and its preprocesor definitions         */

#define EVABBELL        0
#define EVABCAP         1
#define EVABQUICK       2
#define EVACOUNT        3
#define EVASAVE         4
#define EVBUFHOOK       5
#define EVCBFLAGS       6
#define EVCBUFNAME      7
#define EVCFNAME        8
#define EVCMDHK         9
#define EVCMODE         10
#define EVCURCHAR       11
#define EVCURCOL        12
#define EVCURLINE       13
#define EVCURWIDTH      14
#define EVCURWIND       15
#define EVCWLINE        16
#define EVDEBUG         17
#define EVDESKCLR       18
#define EVDIAGFLAG      19
#define EVDISCMD        20
#define EVDISINP        21
#define EVDISPHIGH      22
#define EVDISPUNDO      23
#define EVEXBHOOK       24
#define EVEXITHOOK      25
#define EVFCOL          26
#define EVFILLCOL       27
#define EVFLICKER       28
#define EVFMTLEAD       29
#define EVGFLAGS        30
#define EVGMODE         31
#define EVHARDTAB       32
#define EVHILITE        33
#define EVHILIGHT1      34
#define EVHILIGHT2      35
#define EVHILIGHT3      36
#define EVHILIGHT4      37
#define EVHJUMP         38
#define EVHSCRLBAR      39
#define EVHSCROLL       40
#define EVISTERM        41
#define EVKILL          42
#define EVLANG          43
#define EVLASTKEY       44
#define EVLASTMESG      45
#define EVLINE          46
#define EVLTERM         47
#define EVLWIDTH        48
#define EVMATCH         49
#define EVMMOVE         50
#define EVMODEFLAG      51
#define EVMSFLAG        52
#define EVNEWSCRN       53
#define EVNUMWIND       54
#define EVORGCOL        55
#define EVORGROW        56
#define EVOS            57
#define EVOVERLAP       58
#define EVPAGELEN       59
#define EVPALETTE       60
#define EVPARALEAD      61
#define EVPENDING       62
#define EVPOPFLAG       63
#define EVPOPWAIT       64
#define EVPOSFLAG       65
#define EVPROGNAME      66
#define EVRAM           67
#define EVREADHK        68
#define EVREGION        69
#define EVREPLACE       70
#define EVRVAL          71
#define EVSCRNAME       72
#define EVSEARCH        73
#define EVSEARCHPNT     74
#define EVSEED          75
#define EVSHLHOOK       76
#define EVSOFTTAB       77
#define EVSRES          78
#define EVSSAVE         79
#define EVSSCROLL       80
#define EVSTATUS        81
#define EVSTERM         82
#define EVSYNTAX        83
#define EVTARGET        84
#define EVTIME          85
#define EVTIMEFLAG      86
#define EVTPAUSE        87
#define EVUNDOFLAG      88
#define EVVERSION       89
#define EVVSCRLBAR      90
#define EVWCHARS        91
#define EVWLINE         92
#define EVWRAPHK        93
#define EVWRITEHK       94
#define EVXPOS          95
#define EVYANKFLAG      96
#define EVYPOS          97
#if QDOS
#define EVBEEP          98
#define EVBOLDOFF       99
#define EVBOLDON        100
#define EVCMDINK        101
#define EVCMDPAPER      102
#define EVCRLF          103
#define EVDICTFILE      104
#define EVDURATION      105
#define EVITALOFF       106
#define EVITALON        107
#define EVMLINK         108
#define EVMLPAPER       109
#define EVPAGEFLAG      110
#define EVPAGESIZE      111
#define EVPITCH         112
#define EVPRINTER       113
#define EVPRTINIT       114
#define EVTITLE         115
#define EVTRANSLAT      116
#define EVUNDEROFF      117
#define EVUNDERON       118
#endif

/*      list of recognized user functions       */

typedef struct UFUNC {
        char *f_name;  /* name of function */
        int f_type;     /* 1 = monamic, 2 = dynamic */
} UFUNC;

#define NILNAMIC        0
#define MONAMIC         1
#define DYNAMIC         2
#define TRINAMIC        3

NOSHARE UFUNC funcs[] = {
        "abbrev", MONAMIC,      /* look up abbreviation */
        "abs", MONAMIC,         /* absolute value of a number */
        "add", DYNAMIC,         /* add two numbers together */
        "and", DYNAMIC,         /* logical and */
        "ascii", MONAMIC,       /* char to integer conversion */
        "attr",NILNAMIC,        /* attributes value of the current character */
        "band", DYNAMIC,        /* bitwise and   9-10-87  jwm */
        "bind", MONAMIC,        /* loopup what function name is bound to a key */
        "bnot", MONAMIC,        /* bitwise not */
        "bor", DYNAMIC,         /* bitwise or    9-10-87  jwm */
        "bxor", DYNAMIC,        /* bitwise xor   9-10-87  jwm */
        "call", MONAMIC,        /* call a procedure */
        "cat", DYNAMIC,         /* concatenate string */
        "chr", MONAMIC,         /* integer to char conversion */
        "divide", DYNAMIC,      /* division */
        "env", MONAMIC,         /* retrieve a system environment var */
        "equal", DYNAMIC,       /* logical equality check */
        "exist", MONAMIC,       /* check if a file exists */
        "find", MONAMIC,        /* look for a file on the path... */
        "greater", DYNAMIC,     /* logical greater than */
        "group", MONAMIC,       /* return group match in MAGIC mode */
        "gtc", NILNAMIC,        /* get 1 emacs command */
        "gtk", NILNAMIC,        /* get 1 charater */
        "indirect", MONAMIC,    /* evaluate indirect value */
        "isnum", MONAMIC,       /* is the arg a number? */
        "left", DYNAMIC,        /* left string(string, len) */
        "length", MONAMIC,      /* string length */
        "less", DYNAMIC,        /* logical less than */
        "lower", MONAMIC,       /* lower case string */
        "mid", TRINAMIC,        /* mid string(string, pos, len) */
        "mkcol", MONAMIC,       /* column position of a mark */
        "mkline", MONAMIC,      /* line number of a mark */
        "modulo", DYNAMIC,      /* mod */
        "negate", MONAMIC,      /* negate */
        "not", MONAMIC,         /* logical not */
        "or",  DYNAMIC,         /* logical or */
        "reverse", MONAMIC,     /* reverse */
        "right", DYNAMIC,       /* right string(string, pos) */
        "rnd", MONAMIC,         /* get a random number */
        "sequal", DYNAMIC,      /* string logical equality check */
        "sgreater", DYNAMIC,    /* string logical greater than */
        "sindex", DYNAMIC,      /* find the index of one string in another */
        "sless", DYNAMIC,       /* string logical less than */
        "slower", DYNAMIC,      /* set lower to upper char translation */
        "subtract", DYNAMIC,    /* subtraction */
        "supper", DYNAMIC,      /* set upper to lower char translation */
        "times", DYNAMIC,       /* multiplication */
        "trim", MONAMIC,        /* trim whitespace off the end of a string */
        "true", MONAMIC,        /* Truth of the universe logical test */
        "upper", MONAMIC,       /* uppercase string */
        "xlate", TRINAMIC       /* XLATE character string translation */
};

#define NFUNCS  sizeof(funcs) / sizeof(UFUNC)

/*      and its preprocesor definitions         */

#define UFABBREV        0
#define UFABS           1
#define UFADD           2
#define UFAND           3
#define UFASCII         4
#define UFATTR          5
#define UFBAND          6
#define UFBIND          7
#define UFBNOT          8
#define UFBOR           9
#define UFBXOR          10
#define UFCALL          11
#define UFCAT           12
#define UFCHR           13
#define UFDIV           14
#define UFENV           15
#define UFEQUAL         16
#define UFEXIST         17
#define UFFIND          18
#define UFGREATER       19
#define UFGROUP         20
#define UFGTCMD         21
#define UFGTKEY         22
#define UFIND           23
#define UFISNUM         24
#define UFLEFT          25
#define UFLENGTH        26
#define UFLESS          27
#define UFLOWER         28
#define UFMID           29
#define UFMKCOL         30
#define UFMKLINE        31
#define UFMOD           32
#define UFNEG           33
#define UFNOT           34
#define UFOR            35
#define UFREVERSE       36
#define UFRIGHT         37
#define UFRND           38
#define UFSEQUAL        39
#define UFSGREAT        40
#define UFSINDEX        41
#define UFSLESS         42
#define UFSLOWER        43
#define UFSUB           44
#define UFSUPPER        45
#define UFTIMES         46
#define UFTRIM          47
#define UFTRUTH         48
#define UFUPPER         49
#define UFXLATE         50
